#version 330

layout (points) in;
layout (points) out;
layout(max_vertices = 40) out;

in vec3 outPosition[];
in vec3 outVelocity[];
in float outLifetime[]; 
in int outType[];

uniform float timedelta;
uniform int generate;
uniform vec3 startPosition = vec3(0.0, 97.0, 125.0);

uniform vec3 minPosition = vec3(-31.0, -473.0, -170.0);
uniform vec3 maxPosition = vec3(240.0, 92.0, 270.0);

uniform int velocityMin = -1;
uniform int velocityRange = -4;
uniform float lifetimeMin = 2;
uniform float lifetimeRange = 10;
uniform vec3 randomSeed;

out vec3 positionOut;
out vec3 velocityOut;
out float lifetimeOut;
out int typeOut;

vec3 LocalSeed; 

float randZeroOne()
{
    uint n = floatBitsToUint(LocalSeed.y * 214013.0 + LocalSeed.x * 2531011.0 + LocalSeed.z * 141251.0);
    n = n * (n * n * 15731u + 789221u);
    n = (n >> 9u) | 0x3F800000u;
 
    float fRes =  2.0 - uintBitsToFloat(n);
    LocalSeed = vec3(LocalSeed.x + 147158.0 * fRes, LocalSeed.y*fRes  + 415161.0 * fRes, LocalSeed.z + 324154.0*fRes);
    return fRes;
}


void main()
{
	LocalSeed = randomSeed;

  	positionOut = outPosition[0];
	velocityOut = outVelocity[0];

	if(outType[0] != 0) positionOut += velocityOut * timedelta;
	lifetimeOut = outLifetime[0] - timedelta;
	typeOut = outType[0];

	if (typeOut == 0)
	{
		EmitVertex();
		EndPrimitive();	

		for (int i = 0; i < generate; i++)
		{
			//positionOut = startPosition;

			positionOut = vec3(
				minPosition.x + (randZeroOne() * (maxPosition.x - minPosition.x)),
				minPosition.y + (randZeroOne() * (maxPosition.y - minPosition.y)),
				minPosition.z + (randZeroOne() * (maxPosition.z - minPosition.z))
			 );

			velocityOut = vec3(0.0, velocityMin + velocityRange * randZeroOne(), 0.0);
			lifetimeOut = lifetimeMin + lifetimeRange;

			typeOut = 1;
			EmitVertex();
			EndPrimitive();	
		}
	}
	else if (lifetimeOut > 0.0) 
	{
		EmitVertex();
		EndPrimitive();	
	}
}
