#version 430 core

in vec3 worldNormal;
in vec2 fragmentUV;
in vec3 vertPosition;
in vec4 smPosition[3];
in float ClipSpacePosZ;

const vec3 ambientColor = vec3(0.02, 0.02, 0.02);
const vec3 diffuseColor = vec3(0.2, 0.2, 0.2);
const vec3 specColor = vec3(1.0, 1.0, 1.0);
const float shininess = 100.0;

out vec4 fragColor;

layout(binding = 0) uniform sampler2D color_texture;
layout(binding = 1) uniform sampler2DShadow depth_map[3];
uniform vec3 viewProjectionCenters[3];
uniform vec3 viewProjectionSizes[3];
uniform mat4 model;
uniform vec3 viewPosition;
uniform float transparency;

uniform int showCascades;

const mat4 biasMatrix = mat4(0.5, 0.0, 0.0, 0.0,
							0.0, 0.5, 0.0, 0.0,
							0.0, 0.0, 0.5, 0.0,
							0.5, 0.5, 0.5, 1.0);

void main() {

	vec3 worldPosition = vec3(model * vec4(vertPosition, 1.0));	
	
	vec3 normal = transpose(inverse(mat3(model))) * worldNormal;
	normal = normalize(normal);

  	vec3 lightDir = vec3(0, 1, 0);

  	float lambertian = max(dot(lightDir,normal), 0.0);
  	float specular = 0.0;
    vec3 viewDir = normalize(viewPosition - vertPosition);
	
	if (lambertian > 0.0) {
    		vec3 halfDir = normalize(lightDir + viewDir);
    		float specAngle = max(dot(halfDir, normal), 0.0);
    		specular = pow(specAngle, shininess);
	}

	float shadowFactor = 1.0;
	vec3 indicator = vec3(1.0, 1.0, 1.0);
	for (int i = 0 ; i < 3 ; i++) {
		vec3 minBound = viewProjectionCenters[i] - viewProjectionSizes[i];
		vec3 maxBound = viewProjectionCenters[i] + viewProjectionSizes[i];
		if (all(lessThanEqual(minBound, worldPosition)) && all(greaterThanEqual(maxBound, worldPosition))) {
			if (i == 0) {
				indicator = vec3(1.0, 0.0, 0.0);
			}
			if (i == 1) {
				indicator = vec3(0.0, 1.0, 0.0);
			}
			if (i == 2) {
				indicator = vec3(0.0, 0.0, 1.0);
			}
            shadowFactor = textureProj(depth_map[i], biasMatrix * smPosition[i]);
            break;
        }
	}
	
    //float Depth = textureProj(depth_map[0], smPosition[0]); 

	//fragColor =  vec4(vec3(fragmentUV, 0.0) + ambientColor +
      //               lambertian * diffuseColor * shadowFactor +
      //               specular * specColor * shadowFactor, 1.0);
	if (showCascades == 1) {
		fragColor =  vec4(texture(color_texture, fragmentUV).rgb * indicator + ambientColor +
					lambertian * diffuseColor * shadowFactor +
					specular * specColor * shadowFactor, transparency);
	} else {
		fragColor =  vec4(texture(color_texture, fragmentUV).rgb + ambientColor +
					lambertian * diffuseColor * shadowFactor +
					specular * specColor * shadowFactor, transparency);
	}
}